(function() {
  var coffeeResque, delayedPush, enqueueAt, enqueueIn, getCallbacksObject, identifier, inspect, puts, redisRetryKey, workerPrototype, _ref;
  var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  }, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; }, __indexOf = Array.prototype.indexOf || function(item) {
    for (var i = 0, l = this.length; i < l; i++) {
      if (this[i] === item) return i;
    }
    return -1;
  };
  _ref = require('sys'), puts = _ref.puts, inspect = _ref.inspect;
  coffeeResque = require('coffee-resque');
  exports.watcher = require('./scheduled-task-watcher');
  exports.createWorker = function(connection, queues, jobsWithRetry) {
    var callbacks;
    callbacks = getCallbacksObject(jobsWithRetry);
    return new exports.WorkerWithRetry(connection, queues, callbacks, jobsWithRetry);
  };
  workerPrototype = coffeeResque.Worker.prototype;
  exports.WorkerWithRetry = (function() {
    __extends(WorkerWithRetry, coffeeResque.Worker);
    function WorkerWithRetry(connection, queues, jobs, jobsWithRetry) {
      this.jobsWithRetry = jobsWithRetry;
      coffeeResque.Worker.apply(this, [connection, queues, jobs]);
    }
    WorkerWithRetry.prototype.end = function(cb) {
      this.removeAllListeners('poll');
      this.removeAllListeners('job');
      this.removeAllListeners('error');
      this.removeAllListeners('success');
      return WorkerWithRetry.__super__.end.call(this, cb);
    };
    WorkerWithRetry.prototype.perform = function(job) {
      var key;
      if (!this.running) {
        return;
      }
      key = redisRetryKey(job);
      return this.redis.setnx(key, -1, __bind(function(err, res) {
        if (!((err != null) || !this.running)) {
          return this.redis.incr(key, __bind(function(err, res) {
            if (!((err != null) || !this.running)) {
              return workerPrototype.perform.apply(this, [job]);
            }
          }, this));
        }
      }, this));
    };
    WorkerWithRetry.prototype.succeed = function(result, job) {
      if (!this.running) {
        return;
      }
      return this.redis.del(redisRetryKey(job, __bind(function(err, res) {
        if (!((err != null) || !this.running)) {
          return workerPrototype.succeed.apply(this, [result, job]);
        }
      }, this)));
    };
    WorkerWithRetry.prototype.fail = function(error, job) {
      if (!this.running) {
        return;
      }
      return this.getRetryAttempt(job, __bind(function(err, retryAttempt) {
        var key, limit, _ref;
        if (!((err != null) || !this.running)) {
          limit = ((_ref = this.jobsWithRetry[job["class"]]) != null ? _ref.retry_limit : void 0) || 0;
          if (retryAttempt < limit) {
            return this.tryAgain(job);
          } else {
            key = redisRetryKey(job);
            return this.redis.del(key, __bind(function(err, res) {
              if (!((err != null) || !this.running)) {
                return workerPrototype.fail.apply(this, [error, job]);
              }
            }, this));
          }
        }
      }, this));
    };
    WorkerWithRetry.prototype.tryAgain = function(job) {
      var retryDelay, _ref;
      retryDelay = ((_ref = this.jobsWithRetry[job["class"]]) != null ? _ref.retry_delay : void 0) || 0;
      if (retryDelay <= 0) {
        return coffeeResque.connect(this.redis).enqueue(this.queue, job["class"], job.args);
      } else {
        return enqueueIn(this.redis, retryDelay, this.queue, job["class"], job.args);
      }
    };
    WorkerWithRetry.prototype.getRetryAttempt = function(job, cb) {
      return this.redis.get(redisRetryKey(job), cb);
    };
    return WorkerWithRetry;
  })();
  getCallbacksObject = function(jobsWithRetry) {
    var func, jobs, name;
    jobs = {};
    for (name in jobsWithRetry) {
      func = jobsWithRetry[name].func;
      if (typeof func === 'function') {
        jobs[name] = func;
      }
    }
    return jobs;
  };
  redisRetryKey = function(job) {
    var args, name;
    name = job["class"];
    args = job.args;
    return ['resque-retry', name].concat(identifier(args)).join(":").replace(/\s/g, '');
  };
  identifier = function(args) {
    var a, types;
    types = ['string', 'number', 'boolean'];
    return ((function() {
      var _i, _len, _ref, _results;
      _results = [];
      for (_i = 0, _len = args.length; _i < _len; _i++) {
        a = args[_i];
        if (_ref = typeof a, __indexOf.call(types, _ref) >= 0) {
          _results.push(a.toString().slice(0, 40));
        }
      }
      return _results;
    })()).slice(0, 4).join('-');
  };
  enqueueIn = function(redis, numberOfSecondsFromNow, queue, jobName, args) {
    var timestamp;
    timestamp = (new Date).valueOf() + numberOfSecondsFromNow * 1000;
    return enqueueAt(redis, timestamp, queue, jobName, args);
  };
  enqueueAt = function(redis, timestamp, queue, jobName, args) {
    return delayedPush(redis, timestamp, {
      queue: queue,
      "class": jobName,
      args: args
    });
  };
  delayedPush = function(redis, timestamp, item) {
    redis.rpush("delayed:" + timestamp, JSON.stringify(item));
    return redis.zadd('delayed_queue_schedule', timestamp, timestamp);
  };
}).call(this);
